import os
from pathlib import Path
from shutil import copyfile

import pkg_resources


def get_path_of_data_dir() -> Path:
    """
    get the path of the package data directory

    :returns:

    """
    file_path: str = pkg_resources.resource_filename("rball", "data")

    return Path(file_path)


def get_path_of_data_file(data_file: str) -> Path:
    """
    get the path of a dat file

    :param data_file: name of the data file
    :type data_file: str
    :returns:

    """
    file_path: Path = get_path_of_data_dir() / data_file

    return file_path


def copy_package_data(data_file: str) -> None:
    """
    copy file from the package data directory
    to the current directory

    :param data_file: the name of the file
    :type data_file: str
    :returns:

    """
    data_file_path: Path = get_path_of_data_file(data_file)
    copyfile(data_file_path, f"./{data_file}")


def get_path_of_log_dir() -> Path:
    """
    return the path of the logging directory

    :returns:

    """
    p: Path = Path("~/.log/rball").expanduser()

    if not p.exists():

        p.mkdir(parents=True)

    return p


def get_path_of_log_file(log_file: str) -> Path:
    """
    returns the path of a log file

    :param log_file: the name of the log file
    :type log_file: str
    :returns:

    """
    return get_path_of_log_dir() / log_file


def get_path_of_user_config() -> Path:
    """
    get the path to the user configuration

    :returns:

    """
    config_path: Path = Path().home() / ".config" / "rball"

    if not config_path.exists():

        config_path.mkdir(parents=True)

    return config_path


__all__ = [
    "get_path_of_data_file",
    "get_path_of_data_dir",
    "get_path_of_user_config",
]



import logging
import logging.handlers as handlers
import os
import re
import sys
from hashlib import sha256
from pathlib import Path

from .package_data import get_path_of_log_dir, get_path_of_log_file

_log_file_names = ["usr.log", "dev.log"]


# adapted from thepipe
# https://github.com/tamasgal/thepipe/

ATTRIBUTES = dict(
    list(
        zip([
            'bold', 'dark', '', 'underline', 'blink', '', 'reverse',
            'concealed'
        ], list(range(1, 9)))))
del ATTRIBUTES['']

ATTRIBUTES_RE = r'\033\[(?:%s)m' % '|'  \
                .join(['%d' % v for v in ATTRIBUTES.values()])

HIGHLIGHTS = dict(
    list(
        zip([
            'on_grey', 'on_red', 'on_green', 'on_yellow', 'on_blue',
            'on_magenta', 'on_cyan', 'on_white'
        ], list(range(40, 48)))))

HIGHLIGHTS_RE = r'\033\[(?:%s)m' % '|'  \
                .join(['%d' % v for v in HIGHLIGHTS.values()])

COLORS = dict(
    list(
        zip([
            'grey',
            'red',
            'green',
            'yellow',
            'blue',
            'magenta',
            'cyan',
            'white',
        ], list(range(30, 38)))))

COLORS_RE = r'\033\[(?:%s)m' % '|'.join(['%d' % v for v in COLORS.values()])

RESET = r'\033[0m'
RESET_RE = r'\033\[0m'


def colored(text, color=None, on_color=None, attrs=None, ansi_code=None):
    """Colorize text, while stripping nested ANSI color sequences.
    Author:  Konstantin Lepa <konstantin.lepa@gmail.com> / termcolor
    Available text colors:
        red, green, yellow, blue, magenta, cyan, white.
    Available text highlights:
        on_red, on_green, on_yellow, on_blue, on_magenta, on_cyan, on_white.
    Available attributes:
        bold, dark, underline, blink, reverse, concealed.
    Example:
        colored('Hello, World!', 'red', 'on_grey', ['blue', 'blink'])
        colored('Hello, World!', 'green')
    """
    if os.getenv('ANSI_COLORS_DISABLED') is None:
        if ansi_code is not None:
            return "\033[38;5;{}m{}\033[0m".format(ansi_code, text)
        fmt_str = '\033[%dm%s'
        if color is not None:
            text = re.sub(COLORS_RE + '(.*?)' + RESET_RE, r'\1', text)
            text = fmt_str % (COLORS[color], text)
        if on_color is not None:
            text = re.sub(HIGHLIGHTS_RE + '(.*?)' + RESET_RE, r'\1', text)
            text = fmt_str % (HIGHLIGHTS[on_color], text)
        if attrs is not None:
            text = re.sub(ATTRIBUTES_RE + '(.*?)' + RESET_RE, r'\1', text)
            for attr in attrs:
                text = fmt_str % (ATTRIBUTES[attr], text)
        return text + RESET
    else:
        return text


def cprint(text, color=None, on_color=None, attrs=None):
    """Print colorize text.
    Author:  Konstantin Lepa <konstantin.lepa@gmail.com> / termcolor
    It accepts arguments of print function.
    """
    print((colored(text, color, on_color, attrs)))


def isnotebook():
    """Check if running within a Jupyter notebook"""
    try:
        shell = get_ipython().__class__.__name__
        if shell == 'ZMQInteractiveShell':
            return True  # Jupyter notebook or qtconsole
        elif shell == 'TerminalInteractiveShell':
            return False  # Terminal running IPython
        else:
            return False  # Other type (?)
    except NameError:
        return False


def supports_color():
    """Checks if the terminal supports color."""
    if isnotebook():
        return True
    supported_platform = sys.platform != 'win32' or 'ANSICON' in os.environ
    is_a_tty = hasattr(sys.stdout, 'isatty') and sys.stdout.isatty()

    if not supported_platform or not is_a_tty:
        return False

    return True


DEFAULT_LOG_COLORS = {
    'DEBUG': 'blue',
    'INFO': 'green',
    'WARNING': 'purple',
    'ERROR': 'red',
    'CRITICAL': 'bold_red',

}


def esc(*x):
    """Create escaped code from format code"""
    return '\033[' + ';'.join(x) + 'm'


# The following coloured log logic is from
# https://github.com/borntyping/python-colorlog
# I dropped some features and removed the Python 2.7 compatibility

ESCAPE_CODES = {'reset': esc('0'), 'bold': esc('01'), 'thin': esc('02')}

COLORS = ['black', 'red', 'green', 'yellow', 'blue', 'purple', 'cyan', 'white']

PREFIXES = [
    # Foreground without prefix
    ('3', ''),
    ('01;3', 'bold_'),
    ('02;3', 'thin_'),

    # Foreground with fg_ prefix
    ('3', 'fg_'),
    ('01;3', 'fg_bold_'),
    ('02;3', 'fg_thin_'),

    # Background with bg_ prefix - bold/light works differently
    ('4', 'bg_'),
    ('10', 'bg_bold_'),
]

for _prefix, _prefix_name in PREFIXES:
    for _code, _name in enumerate(COLORS):
        ESCAPE_CODES[_prefix_name + _name] = esc(_prefix + str(_code))


def parse_colors(sequence):
    """Return escape codes from a color sequence."""
    return ''.join(ESCAPE_CODES[n] for n in sequence.split(',') if n)


class ColoredRecord:
    """
    Wraps a LogRecord, adding named escape codes to the internal dict.
    The internal dict is used when formatting the message (by the PercentStyle,
    StrFormatStyle, and StringTemplateStyle classes).
    """

    def __init__(self, record):
        """Add attributes from the escape_codes dict and the record."""
        self.__dict__.update(ESCAPE_CODES)
        self.__dict__.update(record.__dict__)
        self.__record = record

    def __getattr__(self, name):
        return getattr(self.__record, name)


class ColoredFormatter(logging.Formatter):
    """
    A formatter that allows colors to be placed in the format string.
    Intended to help in creating more readable logging output.
    Based on https://github.com/borntyping/python-colorlog
    """

    def __init__(self,
                 fmt,
                 datefmt=None,
                 style='%',
                 log_colors=None,
                 reset=True,
                 secondary_log_colors=None):
        """
        Set the format and colors the ColouredFormatter will use.
        The ``fmt``, ``datefmt`` and ``style`` args are passed on to the
        ``logging.Formatter`` constructor.
        The ``secondary_log_colors`` argument can be used to create additional
        ``log_color`` attributes. Each key in the dictionary will set
        ``{key}_log_color``, using the value to select from a different
        ``log_colors`` set.
        :Parameters:
        - fmt (str): The format string to use
        - datefmt (str): A format string for the date
        - log_colors (dict):
            A mapping of log level names to color names
        - reset (bool):
            Implicitly append a color reset to all records unless False
        - style ('%' or '{' or '$'):
            The format style to use. (*No meaning prior to Python 3.2.*)
        - secondary_log_colors (dict):
            Map secondary ``log_color`` attributes. (*New in version 2.6.*)
        """
        super(ColoredFormatter, self).__init__(fmt, datefmt, style)

        self.log_colors = (log_colors
                           if log_colors is not None else DEFAULT_LOG_COLORS)
        self.secondary_log_colors = secondary_log_colors
        self.reset = reset

    def format(self, record):
        """Format a message from a record object."""
        record = ColoredRecord(record)
        record.log_color = escape_codes(self.log_colors, record.levelname)

        if self.secondary_log_colors:
            for name, log_colors in self.secondary_log_colors.items():
                color = escape_codes(log_colors, record.levelname)
                setattr(record, name + '_log_color', color)

        message = super(ColoredFormatter, self).format(record)

        if self.reset and not message.endswith(ESCAPE_CODES['reset']):
            message += ESCAPE_CODES['reset']

        return message


def escape_codes(log_colors, level_name):
    """Return escape codes from a ``log_colors`` dict."""
    return parse_colors(log_colors.get(level_name, ""))


def hash_coloured(text):
    """Return a ANSI coloured text based on its hash"""
    ansi_code = int(sha256(text.encode('utf-8')).hexdigest(), 16) % 230
    return colored(text, ansi_code=ansi_code)


def hash_coloured_escapes(text):
    """Return the ANSI hash colour prefix and suffix for a given text"""
    ansi_code = int(sha256(text.encode('utf-8')).hexdigest(), 16) % 230
    prefix, suffix = colored('SPLIT', ansi_code=ansi_code).split('SPLIT')
    return prefix, suffix


class LogFilter(object):
    def __init__(self, level):
        self.__level = level

    def filter(self, logRecord):
        return logRecord.levelno != self.__level


# now create the developer handler that rotates every day and keeps
# 10 days worth of backup
rball_dev_log_handler = handlers.TimedRotatingFileHandler(
    get_path_of_log_file("dev.log"), when="D", interval=1, backupCount=10
)

# lots of info written out

_dev_formatter = logging.Formatter(
    "%(asctime)s | %(name)s | %(levelname)s| %(funcName)s | %(lineno)d | %(message)s",
    datefmt="%Y-%m-%d %H:%M:%S",
)

rball_dev_log_handler.setFormatter(_dev_formatter)
rball_dev_log_handler.setLevel(logging.DEBUG)
# now set up the usr log which will save the info

rball_usr_log_handler = handlers.TimedRotatingFileHandler(
    get_path_of_log_file("usr.log"), when="D", interval=1, backupCount=10
)

rball_usr_log_handler.setLevel(logging.INFO)

# lots of info written out
_usr_formatter = logging.Formatter(
    "%(asctime)s | %(levelname)s | %(message)s", datefmt="%Y-%m-%d %H:%M:%S"
)

rball_usr_log_handler.setFormatter(_usr_formatter)

# now set up the console logger
name = "test"
if supports_color():
    prefix_1, suffix = hash_coloured_escapes(name)
    prefix_2, _ = hash_coloured_escapes(name + 'salt')
else:
    prefix_1, prefix_2, suffix = ('', '', '')

date_str = ''

_console_formatter = ColoredFormatter('[%(log_color)s%(levelname)-8s%(reset)s]'
                                      '%(log_color)s %(message)s',
                                      datefmt="%H:%M:%S",

                                      )

rball_console_log_handler = logging.StreamHandler(sys.stdout)
rball_console_log_handler.setFormatter(_console_formatter)
rball_console_log_handler.setLevel("INFO")

warning_filter = LogFilter(logging.WARNING)


def silence_warnings():
    """
    supress warning messages in console and file usr logs
    """

    rball_usr_log_handler.addFilter(warning_filter)
    rball_console_log_handler.addFilter(warning_filter)


def activate_warnings():
    """
    supress warning messages in console and file usr logs
    """

    rball_usr_log_handler.removeFilter(warning_filter)
    rball_console_log_handler.removeFilter(warning_filter)


def update_logging_level(level):

    rball_console_log_handler.setLevel(level)


def setup_logger(name):

    # A logger with name name will be created
    # and then add it to the print stream
    log = logging.getLogger(name)

    # this must be set to allow debug messages through
    log.setLevel(logging.DEBUG)

    # add the handlers

    log.addHandler(rball_dev_log_handler)

    log.addHandler(rball_console_log_handler)

    log.addHandler(rball_usr_log_handler)

    # we do not want to duplicate teh messages in the parents
    log.propagate = False

    return log



# -*- coding: utf-8 -*-

"""Top-level package for rball."""

from .response_database import ResponseDatabase
from .rballlike import RBallLike
from .utils import GridGenerator


__author__ = """J. Michael Burgess"""
__email__ = "jburgess@mpe.mpg.de"

from ._version import get_versions

__version__ = get_versions()["version"]
del get_versions



from os import replace
from typing import Optional, Iterable, Tuple
import numpy as np
import numba as nb
import stripy
from stripy.spherical import lonlat2xyz

import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import ipyvolume as ipv
import pythreejs

from threeML.utils.OGIP.response import InstrumentResponse

import h5py


from .utils.logging import setup_logger


log = setup_logger(__name__)


class ResponseDatabase:
    def __init__(
        self,
        list_of_matrices: np.ndarray,
        theta: np.ndarray,
        phi: np.ndarray,
        ebounds: np.ndarray,
        monte_carlo_energies: np.ndarray,
    ):
        """

        :param list_of_matrices:
        :type list_of_matrices: np.ndarray
        :param theta:
        :type theta: np.ndarray
        :param phi:
        :type phi: np.ndarray
        :param ebounds:
        :type ebounds: np.ndarray
        :param monte_carlo_energies:
        :type monte_carlo_energies: np.ndarray
        ::
        :returns:

        """

        self._matrices: np.ndarray = list_of_matrices

        self._ebounds: np.ndarray = ebounds

        self._monte_carlo_energies: np.ndarray = monte_carlo_energies

        self._n_grid_points: int = self._matrices.shape[0]

        self._matrix_shape = self._matrices.shape[1:]

        self._occulted_matrix = np.zeros(self._matrix_shape)

        if theta.shape[0] != self._n_grid_points:

            log.error(
                f"theta points ({theta.shape[0]}) not equal to number of matrices ({self._n_grid_points})"
            )

            raise AssertionError()

        self._theta: np.ndarray = theta

        if phi.shape[0] != self._n_grid_points:

            log.error(
                f"phi points ({phi.shape[0]}) not equal to number of matrices ({self._n_grid_points})"
            )

            raise AssertionError()

        self._phi: np.ndarray = phi

        # create the triangulation

        self._generate_triangulation()

        # now intitialize the current matrix

        log.debug(f" setting the current matrix to the first data point")

        self._current_matrix: InstrumentResponse = InstrumentResponse(
            matrix=list_of_matrices[0],
            ebounds=self._ebounds,
            monte_carlo_energies=self._monte_carlo_energies,
        )

        self.interpolate_to_position(0.1, 0.1)

    @property
    def current_response(self) -> InstrumentResponse:

        return self._current_matrix

    @property
    def grid_points(self) -> np.ndarray:

        return self._triangulation.points

    @property
    def n_grid_points(self) -> int:
        """
        the number of grid points in
        the response database
        :returns:

        """
        return self._n_grid_points

    def _generate_triangulation(self) -> None:

        log.debug("generating the triangulation")

        self._triangulation = stripy.spherical.sTriangulation(
            lons=self._phi, lats=self._theta, permute=True, tree=True
        )

    def _transform_to_instrument_coordinates(
        self, ra: float, dec: float
    ) -> Tuple[float]:
        """
        This is a stub function that should take and RA/Dec pair (in degrees)
        and convert it into the coordinate system of the intruments which may
        or may not be in motion.

        The function should return a spherical theta, phi tuple in radian

        :param ra:
        :param dec:

        :returns: (theta, phi) in radian

        """

        return np.deg2rad([dec, ra])

    def interpolate_to_position(self, ra: float, dec: float) -> None:

        theta, phi = self._transform_to_instrument_coordinates(ra, dec)

        # obtain the surrounding matricies and their normalized
        # barycenters

        bbc, tri = self._triangulation.containing_simplex_and_bcc(theta, phi)

        log.debug(f"weights: {bbc[0]}, indices: {tri[0]}")

        matrix = _linear_interpolation(bbc[0], self._matrices[tri[0]])

        # update teh 3ML matrix

        self._current_matrix.replace_matrix(matrix)

        self._current_ra: float = ra
        self._current_dec: float = dec

    @property
    def current_sky_position(self) -> Tuple[float]:
        return self._current_ra, self._current_dec

    def plot_verticies(self) -> plt.Figure:

        fig, ax = plt.subplots(subplot_kw=dict(projection="3d"))

        points = self._triangulation.points
        segs = self._triangulation.identify_segments()

        ax.scatter(
            points[:, 0], points[:, 1], points[:, 2], color="k", alpha=0.1
        )

        # plot the verticies

        for s1, s2 in segs:

            ax.plot(
                [points[s1, 0], points[s2, 0]],
                [points[s1, 1], points[s2, 1]],
                [points[s1, 2], points[s2, 2]],
                color="grey",
                alpha=0.5,
            )

        xyzlim = np.array([ax.get_xlim3d(), ax.get_ylim3d(), ax.get_zlim3d()]).T
        XYZlim = [min(xyzlim[0]), max(xyzlim[1])]
        ax.set_xlim3d(XYZlim)
        ax.set_ylim3d(XYZlim)
        ax.set_zlim3d(XYZlim)
        ax.set_box_aspect(
            [
                ub - lb
                for lb, ub in (getattr(ax, f"get_{a}lim")() for a in "xyz")
            ]
        )

        return fig

    def plot_verticies_ipv(
        self, selected_location: Optional[Iterable[float]] = None
    ):
        """

        :param selected_location: ra, dec tuple in degree

        """

        background_color = "#2E0E49"
        grid_color = "#FDFF8D"
        grid_color2 = "#FDFEAE"
        selected_color = "#FF6563"
        point_color = "#63FFA3"

        fig = ipv.figure(width=800, height=600)
        ipv.pylab.style.box_off()
        # ipv.pylab.style.axes_off()
        ipv.pylab.style.set_style_dark()
        ipv.pylab.style.background_color(background_color)

        points = self._triangulation.points
        segs = self._triangulation.identify_segments()

        scatter = ipv.pylab.scatter(
            points[:, 0],
            points[:, 1],
            points[:, 2],
            color=grid_color,
            alpha=1,
            size=1,
            color_selected="red",
            marker="sphere",
        )

        # scatter.selected = tri[0]

        for s1, s2 in segs:
            ipv.plot(
                [points[s1, 0], points[s2, 0]],
                [points[s1, 1], points[s2, 1]],
                [points[s1, 2], points[s2, 2]],
                color=grid_color2,
                alpha=0.5,
            )

        if selected_location is not None:

            theta, phi = self._transform_to_instrument_coordinates(
                *selected_location
            )

            this_point = lonlat2xyz(phi, theta)

            bbc, tri = self._triangulation.containing_simplex_and_bcc(
                phi, theta
            )

            ipv.scatter(
                points[tri, 0],
                points[tri, 1],
                points[tri, 2],
                color=selected_color,
                marker="sphere",
            )

            ipv.pylab.scatter(*this_point, color=point_color, marker="sphere")

        ipv.xyzlim(1.1)
        ipv.pylab.style.box_off()

        fig.camera.up = [0, 0, 1]
        control = pythreejs.OrbitControls(controlling=fig.camera)
        fig.controls = control
        control.autoRotate = True
        fig.render_continuous = True

        ipv.show()


#        return fig


# @nb.njit(fastmath=True)
def _linear_interpolation(
    weights: np.ndarray, super_matrix: np.ndarray
) -> np.ndarray:

    shape = super_matrix[0].shape

    out = np.zeros((shape[0], shape[1]))

    for i in range(3):

        out += weights[i] * super_matrix[i]

    return out



import numpy as np
from stripy.spherical_meshes import icosahedral_mesh


class GridGenerator:
    def __init__(self, refinement_levels: int = 2):

        """
        Generate a grid of points for response generation.
        An Icosahedral with facepoint is chosen because it is closest
        to equal area while still being very fast for triangulation

        :param refinement_levels:
        :type refinement_levels: int
        :returns:

        """
        self._mesh = icosahedral_mesh(
            refinement_levels=refinement_levels, include_face_points=True
        )

    @property
    def n_grid_points(self) -> int:

        return len(self._mesh.lons)

    @property
    def lons(self) -> np.ndarray:

        return self._mesh.lons

    @property
    def lats(self) -> np.ndarray:

        return self._mesh.lats

    @property
    def phi(self) -> np.ndarray:

        return self._mesh.lons

    @property
    def theta(self) -> np.ndarray:

        return self._mesh.lats

    @property
    def xyz(self) -> np.ndarray:

        """
        return the cartesian points on the grid

        :returns:

        """
        return self._mesh.points



from typing import Optional

from threeML.plugins.DispersionSpectrumLike import DispersionSpectrumLike
from threeML.plugins.OGIPLike import OGIPLike
from astromodels import Model

from astromodels.functions.priors import Cosine_Prior, Uniform_prior

from .response_database import ResponseDatabase

from .utils.logging import setup_logger
from rball import response_database


log = setup_logger(__name__)


class RBallLike(DispersionSpectrumLike):
    def __init__(
        self,
        name: str,
        observation,
        response_database: ResponseDatabase,
        background=None,
        free_position: bool = True,
        **kwargs,
    ):

        self._free_position: bool = free_position

        # we replace the response of the observation

        self._response_database: ResponseDatabase = response_database

        observation._response = response_database.current_response

        super(RBallLike, self).__init__(name, observation, background, **kwargs)

    def set_model(self, likelihood_model: Model) -> None:
        """
        Set the model and free the location parameters


        :param likelihood_model:
        :return: None
        """

        # set the standard likelihood model

        super(RBallLike, self).set_model(likelihood_model)

        # now free the position
        # if it is needed

        if self._free_position:

            log.info(f"freeing the position of {self.name} and setting priors")

            for key in self._like_model.point_sources.keys():
                self._like_model.point_sources[key].position.ra.free = True
                self._like_model.point_sources[key].position.dec.free = True

                self._like_model.point_sources[
                    key
                ].position.ra.prior = Uniform_prior(
                    lower_bound=0.0, upper_bound=360
                )
                self._like_model.point_sources[
                    key
                ].position.dec.prior = Cosine_Prior(
                    lower_bound=-90.0, upper_bound=90
                )

                ra = self._like_model.point_sources[key].position.ra.value
                dec = self._like_model.point_sources[key].position.dec.value

        else:

            for key in self._like_model.point_sources.keys():

                self._like_model.point_sources[
                    key
                ].position.ra.prior = Uniform_prior(
                    lower_bound=0.0, upper_bound=360
                )
                self._like_model.point_sources[
                    key
                ].position.dec.prior = Cosine_Prior(
                    lower_bound=-90.0, upper_bound=90
                )

                ra = self._like_model.point_sources[key].position.ra.value
                dec = self._like_model.point_sources[key].position.dec.value

        self._response_database.interpolate_to_position(ra, dec)

    def get_model(self, precalc_fluxes=None):

        # Here we update the GBM drm parameters which creates and new DRM for that location
        # we should only be dealing with one source for GBM

        # update the location

        if self._free_position:

            # assumes that the is only one point source which is how it should be!
            ra, dec = self._like_model.get_point_source_position(0)

            self._response_database.interpolate_to_position(ra, dec)

        return super().get_model(precalc_fluxes)

    @classmethod
    def from_spectrumlike(
        cls,
        spectrum_like: DispersionSpectrumLike,
        response_database: ResponseDatabase,
        free_position: bool = True,
        **kwargs,
    ):
        """
        Generate a RBallLike from an existing SpectrumLike child


        :param spectrum_like: the existing spectrumlike
        :param response_database:  a response database
        :param free_position: if the position should be free
        :return:
        """

        try:

            observation = spectrum_like._observed_spectrum.to_binned_spectrum()

        except:

            observation = spectrum_like._observed_spectrum

        try:

            bkg = spectrum_like._background_spectrum.to_binned_spectrum()

        except:

            bkg = spectrum_like._background_spectrum

        return cls(
            spectrum_like.name,
            observation,
            response_database,
            bkg,
            free_position,
            **kwargs,
        )

    @classmethod
    def from_ogip(
        cls,
        name: str,
        observation: str,
        response_database: ResponseDatabase,
        background: Optional[str] = None,
        spectrum_number: Optional[int] = None,
        free_position: bool = True,
        **kwargs,
    ):
        """
        Create an RBallLike from OGIP files

        :param cls:
        :type cls:
        :param name:
        :type name: str
        :param observation:
        :type observation: str
        :param background:
        :type background: str
        :param response_database:
        :type response_database: ResponseDatabase
        :param spectrum_number:
        :type spectrum_number: Optional[int]
        :param free_position:
        :type free_position: bool
        :returns:

        """
        tmp = OGIPLike(
            name=name,
            observation=observation,
            background=background,
            response=response_database.current_response,
            spectrum_number=spectrum_number,
        )

        return cls.from_spectrumlike(
            tmp,
            response_database=response_database,
            free_position=free_position,
            **kwargs,
        )

    def get_simulated_dataset(self, new_name=None, **kwargs):
        return super().get_simulated_dataset(
            new_name=new_name,
            response_database=self._response_database,
            **kwargs,
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os

import versioneer
from setuptools import setup

# # Create list of data files
# def find_data_files(directory):

#     paths = []

#     for (path, directories, filenames) in os.walk(directory):

#         for filename in filenames:

#             paths.append(os.path.join("..", path, filename))

#     return paths


#extra_files = find_data_files("rball/data")

setup(

    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    #        package_data={"": extra_files},
)
